<?php

/**
 * @file
 * Admin pages callbacks for the registrar_api module.
 */

/**
 * Callback function for the registrar_api admin settings form.
 */
function registrar_api_settings_form() {
  $form = array();

  // Select available TLDs
  $tlds = variable_get('registrar_api_iana_tlds', array());
  $form['iana'] = array(
    '#type' => 'fieldset',
    '#title' => t('Available TLDs'),
    '#collapsed' => TRUE,
    '#collapsible' => TRUE,
  );

  if ($tlds) {
    $form['iana']['#description'] = t('Select the supported TLDs');
  }
  else {
    $form['iana']['#collapsed'] = FALSE;
    $form['iana']['msg'] = array(
      '#value' => t('To start using registrar API it\'s needed to download the TLDs database from <a href="@url">@url</a>', array('@url' => REGISTRAR_API_IANA_TLDS_URL)),
      '#prefix' => '<p>',
      '#suffix' => '</p>',
    );
    $form['iana']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Download IANA database now'),
      '#button_type' => 'iana-submit',
    );
    $form['#submit'] = array('registrar_api_settings_initialize_tlds');
    return $form;
  }

  $tlds_enabled = variable_get('registrar_api_iana_tlds_enabled', array());
  if (!empty($tlds_enabled)) {
    $form['iana']['#collapsed'] = TRUE;
  }
  $options = array_merge($tlds_enabled, $tlds);
  $form['iana']['tlds'] = array(
    '#type' => 'checkboxes',
    '#options' => $tlds,
    '#value' => $tlds_enabled,
  );

  // Select default registrar and associate to each TLD.
  $registrars = registrar_api_get_registrars();
  if (empty($registrars)) {
    drupal_set_message(t('You must enable a <a href="@modules">registrar module</a> to make use of the registrar_api.', array('@modules' => url('admin/build/modules'))), 'warning');
  }
  else {
    $form['registrars'] = array(
      '#type' => 'fieldset',
      '#title' => t('Select registrars'),
      '#description' => t('Select the registrar to use by default and optionally a specific registrar per TLD.'),
      '#collapsible' => TRUE,
    );
    $options = array();
    foreach ($registrars as $k => $v) {
      $options[$k] = $v['name'];
    }
    $default = variable_get('registrar_api_default_registrar', '');
    if ($default != '') {
      $form['registrars']['#collapsed'] = TRUE;
    }
    $form['registrars']['default_registrar'] = array(
      '#type' => 'select',
      '#title' => t('Default'),
      '#default_value' => $default,
      '#options' => $options,
    );

    $form['registrars']['tld_registrar'] = array(
      '#type' => 'item',
      '#tree' => TRUE,
    );
    $options = array_merge(array('' => t('Default')), $options);
    foreach ($tlds_enabled as $tld) {
      $items = array(
        'tld' => array('#value' => $tld),
        'registrar' => array(
          '#type' => 'select',
          '#options' => $options,
          '#default_value' => variable_get('registrar_api_registrar_'.$tld, ''),
        ),
      );
      $form['registrars']['tld_registrar'][$tld] = $items;
    }
  }

  // Default nameservers.
  $form['nameservers'] = array(
    '#type' => 'fieldset',
    '#title' => t('Default nameservers'),
    '#description' => t('Select the nameservers to use by default.'),
    '#collapsible' => TRUE,
    '#tree' => TRUE,
  );
  $nameservers = variable_get('registrar_api_nameservers_default', array());
  if (!empty($nameservers)) {
    $form['nameservers']['#collapsed'] = TRUE;
  }
  $total = count($nameservers) + 3;
  for ($i = 0; $i < $total; $i++) {
    $form['nameservers'][$i] = array(
      '#type' => 'textfield',
      '#default_value' => isset($nameservers[$i]) ? $nameservers[$i] : '',
    );
  }

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save configuration')
  );

  return $form;
}

/**
 * Validate callback.
 */
function registrar_api_settings_form_validate($form, &$form_state) {
  if ($form_state['clicked_button']['#button_type'] == 'iana-submit') {
    // No validation necessary for the iana-submit button.
    return;
  }

  // Validate nameservers.
  foreach ($form_state['values']['nameservers'] as $i => $value) {
    if (empty($value)) {
      unset($form_state['values']['nameservers'][$i]);
    }
    elseif (!_registrar_api_check_valid_nameserver($value)) {
      form_error($form['nameservers'][$i], t('Invalid nameserver.'));
    }
  }
}

/**
 * Submit callback.
 */
function registrar_api_settings_form_submit($form, $form_state) {
  // Enabled IANA TLDs.
  $tlds = variable_get('registrar_api_iana_tlds', array());
  $enabled = array_intersect_assoc($tlds, $form_state['values']['tlds']);
  variable_set('registrar_api_iana_tlds_enabled', $enabled);

  // Default and per tld registrars.
  variable_set('registrar_api_default_registrar', $form_state['values']['default_registrar']);
  if (isset($form_state['values']['tld_registrar'])) {
    if ($tld_registrar = $form_state['values']['tld_registrar']) {
      foreach ($tld_registrar as $tld => $value) {
        variable_set('registrar_api_registrar_' . $tld, $value['registrar']);
      }
    }
  }

  // Default nameservers.
  variable_set('registrar_api_nameservers_default', array_values($form_state['values']['nameservers']));
}

/**
 * Submit callback for the settings form when the list of TLDs is not available.
 */
function registrar_api_settings_initialize_tlds($form, &$form_state) {
  $result = drupal_http_request(REGISTRAR_API_IANA_TLDS_URL);
  if ($result->code != 200) {
    $msg = t('There was a problem trying to download the TLDs database: "!error"', array('!error' => $result->error));
    form_error($form['iana'], $msg);
  }
  else {
    $tlds = explode("\n", trim($result->data));
    array_shift($tlds);
    $tlds = drupal_map_assoc($tlds);
    variable_set('registrar_api_iana_tlds', $tlds);
    drupal_set_message('TLDs database imported succesfully.');
  }
}

/**
 * Return the form for admin/settings/registrar_api/registrars/{RNAME}.
 *
 * Load the registrar specific form and extend it with items to set
 * default contacts' handlers.
 */
function registrar_api_add_contacts_form(&$form, &$form_state, $rname) {
  $form['registrar_api']['registrar'] = array(
    '#type' => 'hidden',
    '#value' => $rname,
  );

  $form['registrar_api']['handles'] = array(
    '#type' => 'fieldset',
    '#title' => t('Default contacts'),
    '#description' => t('You can specify default contacts for domains registered with this registrar. If a default contact is choosen for any contact type, users without the «override default contacts» permission will not be able to change that contact type on domain creation.'),
    '#collapsible' => TRUE,
    '#weight' => -10,
  );

  $form['registrar_api']['handles'] += _registrar_api_domain_handles_form('admin');
  $form['#validate'][] = 'registrar_api_contacts_form_validate';
  $form['#submit'][] = 'registrar_api_contacts_form_submit';
}

/**
 * Validate callback.
 */
function registrar_api_contacts_form_validate($form, &$form_state) {
  foreach (registrar_api_contact_types() as $ctype => $name) {
    if ($contact = $form_state['values'][$ctype]) {
      // Check if the contacts exists.
      $nid = _registrar_api_nid_from_autocomplete($contact);
      if (!db_result(db_query("SELECT nid FROM {registrar_api_contact} WHERE nid = %d", $nid))) {
        form_set_error($ctype, t('!name contact %contact does not exist.', array('!name' => $name, '%contact' => $contact)));
      }
    }
  }
}

/**
 * Submit callback.
 */
function registrar_api_contacts_form_submit($form, &$form_state) {
  $rname = $form_state['values']['registrar'];
  foreach (registrar_api_contact_types() as $ctype => $name) {
    $variable = 'registrar_api_' . $rname . '_default_' . $ctype;
    variable_set($variable, $form_state['values'][$ctype]);
  }
}

/**
 * Check if a nameserver is a valid hostname.
 */
function _registrar_api_check_valid_nameserver($ns) {
  // IP addresses are fine.
  // From http://www.blog.highub.com/regular-expression/php-regex-regular-expression/php-regex-validate-ip-address/
  $pattern = "/^(([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5]).){3}([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/";
  if (preg_match($pattern, $ns)) {
    return TRUE;
  }

  // Hostname.
  // @TODO: see RFCs 952 and 1123. Also http://en.wikipedia.org/wiki/Hostname
  if (drupal_valid_http_host($ns)) {
    return TRUE;
  }

  return FALSE;
}
