<?php

/**
 * @file
 * Hooks provided by the registrar api module.
 */

/**
 * @addtogroup hooks
 * @{
 */

/**
 * Provide metadata about the registrar being implemented.
 *
 * @return
 *   An associative array with the registrar details.
 */
function hook_registrar_info() {
  return array(
    'myregistrar' => array(
      'name' => 'MyRegistrar',
      'description' => 'MyRegistrar domain registrar',
    ),
  );
}

/**
 * Check for the availability of the given domains.
 *
 * @param $domain_list
 *   An array containing domain names.
 *
 * @return
 *   Associative array of domains and their status's.
 */
function hook_check_domains($domain_list) {
  $domains = array();
  foreach ($domain_list as $domain) {
    $response = myregistrar_domain_check_api_call($domain);
    $domains[$domain] = $response['Available'];
  }
  return $domains;
}

/**
 * Register a new domain name.
 *
 * @param $domain_name
 *   The domain name to be registered.
 * @param $registrant
 *   Contact details for the registrant contact.
 * @param $tech
 *   Contact details for the tech contact.
 * @param $admin
 *   Contact details for the admin contact.
 * @param $years
 *   Number of years to register the domain for.
 * @param $nameservers
 *   Array of nameserver IP address to attach to the domain.
 *
 * @return
 *   An associative array containingn the success status and the amount
 *   charged.
 */
function hook_create_domain($domain_name, $registrant, $tech, $admin, $years = 1, $nameservers = NULL) {

  $request['DomainName'] = $domain_name;
  $request['Years'] = $years;
  $request['RegistrantInfo'] = $registrant;
  $request['TechInfo'] = $tech;
  $request['AdminInfo'] = $admin;
  $request['Nameservers'] = $nameservers;

  $response = myregistrar_api_request_create($request);
  return $response;
}

/**
 * Register an existing domain name.
 *
 */
function hook_renew_domain($domain_name, $years) {
  $request['DomainName'] = $domain_name;
  $request['Years'] = $years;
  $response = myregistrar_api_request_renew($request);
  return $response;
}

/**
 * Return the list of all contacts for the specified domain name.
 *
 * This would generally be used to either sync up domains already in the system
 * that may have been modified externally or to import data from domains not
 * already in the users local account.
 * 
 * @param $domain_name - The domain to retrieve the contacts for.
 *
 * @return
 *   Associative array containing the domain name and the registrant, tech,
 *   and admin contact information. For example:
 *   array(
 *    'domain' => 'mydomain.com',
 *    'registrant' => array(
 *      'company_name' => 'MyCompany',
 *      'first_name' => 'First',
 *      'last_name' => 'Last',
 *      'location_street' => '1234 Acme Rd.'
 *      'location_additional' => '',
 *      'location_city' => 'Acme City',
 *      'location_province' => 'California',
 *      'location_postal_code' => '90210',
 *      'location_country' => 'us'
 *      'location_phone' => '+1.5555555555',
 *      'location_fax' => '+1.5555555556',
 *      'email' => 'mymail@example.com',
 *    ),
 *    'tech' => array(...),
 *    'admin' => array(...),
 *  );
 * 
 */
function hook_get_contacts($domain_name) {}

/**
 * Set the contact details on the specified domain.
 *
 * @param $domain
 *   The domain name in question.
 * @param $registrant
 *   Contact details for the registrant contact.
 * @param $tech
 *   Contact details for the tech contact.
 * @param $admin
 *   Contact details for the admin contact.
 *
 * @return
 *   Returns TRUE if the update was successful, FALSE otherwise.
 */
function hook_set_contacts($domain, $registrant, $tech, $admin) {}

/**
 * Get the list of all registered domains from the given registrar.
 *
 * @return
 *   An associative array keyed on domain name. The data in the array
 *   associated with each domain name will vary based on the registrar.
 */
function hook_registered_domains() {}


/**
 * Alter the default location settings.
 *
 * @see _registrar_api_location_settings().
 */
function hook_registrar_api_location_settings_alter() {}

/**
 * @} End of "addtogroup hooks".
 */
