<?php

/**
 * @file
 * Provides all of the Registrar API calls.
 */

/**
 * Check the availability status on a list of domains.
 *
 * @param $domain_list
 *   An array containing a list of domains to test availability on
 *
 * @return multi-dimensional array key'd by domain name, for example:
 *   Array (
 *     [domain1.com] => Array (
 *       [Domain] => domain1.com,
 *       [Available] => false,
 *     )
 *   )
 */
function registrar_api_check_domains($domain_list) {
  return _registrar_api_method('check_domains', $domain_list);
}

/**
 * Check the availability status on the specified domain.
 * @uses registrar_api_check_domains()
 *
 * @param $domain
 *   The domain name to check availability on
 *
 * @return TRUE if domain is available, FALSE otherwise
 */
function registrar_api_check_domain($domain) {
  $domain_results = registrar_api_check_domains(array($domain));
  return $domain_results[$domain];
}

/**
 * Get the list of domains currently registered at the default registrar.
 *
 * @return
 *   An associative array keyed on domain name. The data in the array
 *   associated with each domain name will vary based on the registrar.
 *
 * @TODO integrate additional data?
 * @TODO add optional parameters: limit, search pattern?
 */
function registrar_api_registered_domains() {
  return _registrar_api_method('registered_domains');
}

/**
 * Register the given domain name
 *
 * @param $domain_name
 *   The domain name to register
 * @param $registrant
 *   An array containing all of the registrant contact details
 * @param $tech
 *   An array containing all of the tech contact details
 * @param $admin
 *   An array containing all of the admin contact details
 * @param $years
 *   The number of years to register the domain for
 * @param $nameservers
 *   The nameservers to use for the domain
 *
 * return array containing the registered domain information
 */
function registrar_api_create_domain($domain_name, $registrant_id, $tech_id, $admin_id, $billing_id, $years = 1, $nameservers = NULL) {
  $registrant = node_load($registrant_id);
  _registrar_convert_locations($registrant);
  $tech = node_load($tech_id);
  _registrar_convert_locations($tech);
  $admin = node_load($admin_id);
  _registrar_convert_locations($admin);
  $billing = node_load($billing_id);
  _registrar_convert_locations($billing);

  return _registrar_api_method('create_domain', $domain_name, $registrant, $tech, $admin, $billing, $years, $nameservers);
}

/**
 * Check the availability status on the specified domain.
 *
 * @param $domain
 *   The domain name to renew
 *
 * @param $years
 *   The number of years to renew the domain for
 *
 * @return array containing renewal information
 */
function registrar_api_renew_domain($domain, $years = 1) {
  // @TODO: Verify that this user owns the domain being renewed
  return _registrar_api_method('renew_domain', $domain, $years);
}

/**
 * Set the contact information on the domain
 *
 * @param $domain
 *   The domain to be updated
 * @param $registrant
 *   An array containing all of the registrant contact details
 * @param $tech
 *   An array containing all of the tech contact details
 * @param $admin
 *   An array containing all of the admin contact details
 * @param $auxbilling
 *   An array containing all of the billing details
 *
 * @return
 *   TRUE if the update was successful, FALSE otherwise.
 */
function registrar_api_set_contacts($domain, $registrant_id, $tech_id, $admin_id, $billing_id) {
  $registrant = node_load($registrant_id);
  _registrar_convert_locations($registrant);
  $tech = node_load($tech_id);
  _registrar_convert_locations($tech);
  $admin = node_load($admin_id);
  _registrar_convert_locations($admin);
  $billing = node_load($billing_id);
  _registrar_convert_locations($billing);

  if (!($status = _registrar_api_method('set_contacts', $domain, $registrant, $tech, $admin, $billing))) {
    drupal_set_message('Contacts were not updated for %domain.', array('%domain' => $domain), 'warning');
  }

  return $status;
}

/**
 * Get the contact information for a domain
 *
 * @param $domain
 *   The name of the domain to retrieve the contact information for
 *
 * @return array containing the domain contact information
 */
function registrar_api_get_contacts($domain) {
  return _registrar_api_method('get_contacts', $domain);
}

/**
 * Adds locations directly to the node object.
 *
 * This is used to make it easier to reference locations in the individual
 * registrar code bases when contacts need to be passed through the registrar.
 */
function _registrar_convert_locations(&$node) {
  foreach ($node->location[0] as $field => $value) {
    $field = 'location_' . $field;
    $node->$field = $value;
  }
}

function _registrar_api_method() {
  $args = func_get_args();
  $method = array_shift($args);

  if ($registrar = variable_get('registrar_api_default_registrar', '')) {
    $method = 'registrar_' . $registrar . '_' . $method;
    return call_user_func_array($method, $args);
  }
  else {
    drupal_set_message(t('No default registrar set'), 'error');
  }
}
