<?php

/**
 * @file
 * Hooks to implement the custom content types
 */

/**
 * Implementation of hook_node_info().
 *
 * Note 'module' key isn't actually a real module name but
 * the prefix we want for the hooks. It has been renamed to
 * 'base' in Drupal 7.
 */
function registrar_api_node_info() {
  return array(
    'registrar_api_contact' => array(
      'name' => t('Domain Contact Information'),
      'module' => 'registrar_api_contact',
      'description' => 'Contact information used with a domain',
      'has_body' => FALSE,
    ),
    'registrar_api_domain' => array(
      'name' => t('Domain'),
      'module' => 'registrar_api_domain',
      'description' => 'A domain name',
      'title_label' => t('Domain'),
      'has_body' => FALSE,
    ),
  );
}

/**
 * Implementation of hook_form() for contact content type.
 */
function registrar_api_contact_form(&$node, $form_state) {
  $form = array();

  // Title will be auto-generated based on the first and last names.
  $form['title'] = array(
    '#type' => 'value',
    '#value' => $node->title,
  );

  // DB schema fields

  $form['uid'] = array(
    '#type' => 'textfield',
    '#title' => t('Users table uid'),
    '#required' => TRUE,
    '#default_value' => isset($node->uid) ? $node->uid : NULL,
  );

  $form['first_name'] = array(
    '#type' => 'textfield',
    '#title' => t('First name'),
    '#required' => TRUE,
    '#size' => 60,
    '#maxlength' => 60,
    '#default_value' => isset($node->first_name) ? $node->first_name : ''
  );
  $form['last_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Last name'),
    '#required' => TRUE,
    '#size' => 60,
    '#maxlength' => 60,
    '#default_value' => isset($node->last_name) ? $node->last_name : ''
  );
  $form['email'] = array(
    '#type' => 'textfield',
    '#title' => t('Email address'),
    '#required' => TRUE,
    '#size' => 60,
    '#maxlength' => 255,
    '#default_value' => isset($node->email) ? $node->email : ''
  );
  $form['company_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Company Name'),
    '#required' => FALSE,
    '#size' => 60,
    '#maxlength' => 60,
    '#default_value' => isset($node->company_name) ? $node->company_name : ''
  );

  $settings = _registrar_api_location_settings();
  $location = isset($node->location) ? $node->location : array();
  $form['location'] = location_form($settings, $location);

  return $form;
}

/**
 * Implementation of hook_validate().
 */
function registrar_api_contact_validate($node, &$form) {
  // @TODO: Validate uid

  // @TODO: Validate contacts.
  if (!user_access('override default contacts')) {

  }
}

/**
 * Title for registrar contact information is set automatically.
 */
function registrar_api_nodeapi(&$node, $op, $a3 = NULL, $a4 = NULL) {
  if ($node->type == 'registrar_api_contact' && $op == 'presave') {
    if (empty($node->title)) {
      $node->title = $node->first_name . ' ' . $node->last_name;
    }
  }
}

/**
 * Implementation of hook_insert().
 */
function registrar_api_contact_insert($node) {
  drupal_write_record('registrar_api_contact', $node);
  drupal_write_record('registrar_api_user_handles', $node);
  location_save_locations($node->location, array('nid' => $node->nid, 'vid' => $node->vid));
}

/**
 * Implementation of hook_update().
 */
function registrar_api_contact_update($node) {
  if (empty($node->title)) {
    $node->title = $node->first_name . ' ' . $node->last_name;
  }
  drupal_write_record('registrar_api_contact', $node, 'nid');
  drupal_write_record('registrar_api_user_handles', $node, 'nid');
  location_save_locations($node->location, array('nid' => $node->nid, 'vid' => $node->vid));
}

/**
 * Implementation of hook_view().
 */
function registrar_api_contact_view($node, $teaser = FALSE, $page = FALSE) {
  $node->content['Contact'] = array(
    '#value' => theme('registrar_api_contact_info', $node, TRUE),
    '#weight' => 1,
  );
  $settings = _registrar_api_location_settings();
  if (isset($settings['display']['teaser']) && isset($settings['display']['full'])) {
    if (($teaser && $settings['display']['teaser']) || (!$teaser && $settings['display']['full'])) {
      $node->content['location'] = location_display($settings, $node->location);
    }
  }
  return $node;
}

/**
 * Implementation of hook_load().
 */
function registrar_api_contact_load($node) {
  $contact = db_fetch_object(db_query("SELECT * FROM {registrar_api_contact} WHERE nid = %d", $node->nid));
  $contact->location = location_load_locations($node->vid);
  return $contact;
}

/**
 * Implementation of hook_form() for the domain content type.
 */
function registrar_api_domain_form(&$node, $form_state) {
  $type = node_get_types('type', $node);

  /* DB schema fields */
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => check_plain($type->title_label),
    '#required' => TRUE,
    '#default_value' => $node->title,
    '#size' => 20,
  );
  if (isset($node->nid)) {
    $form['title']['#disabled'] = TRUE;
    $form['title']['#value'] = $node->title;
  }

  // Contact handlers.
  $form['handles'] = array(
    '#type' => 'fieldset',
    '#title' => t('Contacts'),
    '#collapsible' => TRUE,
    '#weight' => 5,
  );

  $form['handles'] += _registrar_api_domain_handles_form();

  $form['handles']['create_contacts'] = array(
    '#value' => l(t('Add more contacts'), 'node/add/registrar-api-contact'),
    '#prefix' => '<p>',
    '#suffix' => '</p>',
  );

  if (!isset($node->nid)) {
    $form['register_domain'] = array(
      '#type' => 'checkbox',
      '#title' => t('Register domain'),
      '#description' => t('Uncheck this to only create the node and skip domain validation/creation'),
      '#default_value' => TRUE,
    );
  }

  return $form;
}

/**
 * Implementation of hook_validate().
 *
 * If a handle doesn't exist, show an error.
 * A domain needs those three handles, otherwise we can't register them
 */
function registrar_api_domain_validate($node, &$form) {
  // Validate the domain name
  if (!$node->nid && $node->register_domain && !registrar_api_check_domain($node->title)) {
    form_set_error('title', t('Domain %domain is not available', array('%domain' => $node->title)));
  }

  foreach (registrar_api_contact_types() as $ctype => $name) {
    // Check if the contacts exists.
    $contact = $form['handles'][$ctype]['#value'];
    $nid = _registrar_api_nid_from_autocomplete($contact);
    if (!db_result(db_query("SELECT nid FROM {registrar_api_contact} WHERE nid = %d", $nid))) {
      form_set_error($ctype, t('!name contact %contact does not exist.', array('!name' => $name, '%contact' => $contact)));
    }
  }
}

/**
 * Implementation of hook_insert().
 *
 * This function is also called by registrar_api_domain_update().
 */
function registrar_api_domain_insert($node, $update = array()) {
  $record = array(
    'nid' => $node->nid,
  );
  foreach (registrar_api_contact_types() as $ctype => $name) {
    $nid = _registrar_api_nid_from_autocomplete($node->$ctype);
    $record[$ctype] = $nid;
  }
  drupal_write_record('registrar_api_domain', $record, $update);

  if (registrar_api_check_domain($node->title)) {
    registrar_api_create_domain($node->title, $record['registrant_contact'], $record['tech_contact'], $record['admin_contact'], $record['billing_contact']);
  }
}

/**
 * Implementation of hook_update().
 */
function registrar_api_domain_update($node) {
  // TODO: module_invoke_all('update_contact', $node);
  registrar_api_domain_insert($node, 'nid');
}

function registrar_api_domain_view($node, $teaser = FALSE, $page = FALSE) {
  $node->content['Domain'] = array(
    '#value' => $node->title,
    '#weight' => 1,
  );
  return $node;
}

/**
 * Implementation of hook_load().
 */
function registrar_api_domain_load($node) {
  // Fetch the names iso the id's for the handles
  // Now we can autocomplete the names iso the id's
  // @see registrar_api_domain_update
  // @see registrar_api_domain_insert
  $query = "
  SELECT domain.nid,
    CONCAT(billing_handle.title, ' [nid: ', billing_handle.nid, ']') as billing_contact,
    CONCAT(tech_handle.title, ' [nid: ', tech_handle.nid, ']') AS tech_contact,
    CONCAT(admin_handle.title, ' [nid: ', admin_handle.nid, ']') AS admin_contact,
    CONCAT(registrar_handle.title, ' [nid: ', registrar_handle.nid, ']') as registrant_contact
  FROM {registrar_api_domain} AS domain
  LEFT JOIN ({registrar_api_contact} AS billing_contact, {node} AS billing_handle)
    ON (domain.billing_contact = billing_contact.nid AND billing_contact.nid = billing_handle.nid)
  LEFT JOIN ({registrar_api_contact} AS domain_contact, {node} AS tech_handle)
    ON (domain.tech_contact = domain_contact.nid AND domain_contact.nid = tech_handle.nid)
  LEFT JOIN ({registrar_api_contact} AS admin_contact, {node} AS admin_handle)
    ON (domain.admin_contact = admin_contact.nid AND admin_contact.nid = admin_handle.nid)
  LEFT JOIN ({registrar_api_contact} AS registrar_contact, {node} AS registrar_handle)
    ON (domain.registrant_contact = registrar_contact.nid AND registrar_contact.nid = registrar_handle.nid)
  WHERE domain.nid=%d
  ";
  $domain = db_fetch_object(db_query($query, $node->nid));
  return $domain;
}

/**
 * There is no implementation of hook_delete()
 * Contacts may not be deleted, because they can be used by domains
 * If there are domains active, and the contact is deleted, we may
 * have a problem with renewing the domain.
 *
 * @TODO: Deletions of contacts attached to no domains should be
 *   possible.
 */

/**
 * Helper to obtain default settings for location form.
 *
 * @see hook_registrar_api_location_settings_alter().
 */
function _registrar_api_location_settings() {
  $settings = array(
    'multiple' => array(
      'min' => 1,
      'max' => 1,
      'add' => 1
    ),
    'form' => array(
      'weight' => 1,
      'collapsible' => 0,
      'collapsed' => 0,
      'fields' => array(
        'name' => array(
          'collect' => 0,
          'default' => NULL,
          'weight' => 2
        ),
        'country' => array(
          'collect' => 1,
          'default' => '',
          'weight' => 4
        ),
        'province' => array(
          'collect' => 2,
          'default' => NULL,
          'weight' => 6
        ),
        'city' => array(
          'collect' => 2,
          'default' => NULL,
          'weight' => 8
        ),
        'street' => array(
          'collect' => 2,
          'default' => NULL,
          'weight' => 10
        ),
        'additional' => array(
          'collect' => 1,
          'default' => NULL,
          'weight' => 12
        ),
        'postal_code' => array(
          'collect' => 2,
          'default' => NULL,
          'weight' => 14
        ),
        'locpick' => array(
          'collect' => 0,
          'weight' => 20,
        ),
        'phone' => array(
          'collect' => 2,
          'default' => NULL,
          'weight' => 25
        ),
        'fax' => array(
          'collect' => 1,
          'default' => NULL,
          'weight' => 30
        )
      )
    ),
    'display' => array(
      'weight' => 1,
      'hide' => array(
        'country' => 0,
        'country_name' => 0,
        'province' => 0,
        'province_name' => 0,
        'city' => 0,
        'name' => 0,
        'street' => 0,
        'additional' => 0,
        'postal_code' => 0,
        'locpick' => 0,
        'fax' => 0,
        'phone' => 0,
        'map_link' => 0,
        'coords' => 0,
      ),
      'teaser' => 1,
      'full' => 1,
    ),
    'rss' => array(
      'mode' => FALSE
    )
  );
  drupal_alter('registrar_api_location_settings', $settings);

  return $settings;
}
